%%% Monte Carlo Dynamic Classifier tool - MCDC tool
% http://www.ismvideo.org/NDE0UP/MCDCtool/en/index.html

% Copyright (C) 2014-2015 Gareth W. Peters, Tomoko Matsui, Francois Septier, and Akira Tamamori

% This program is free software; you can redistribute it and/or modify it under the terms
% of the GNU General Public License as published by the Free Software Foundation; either
% version 2 of the License, or (at your option) any later version.

% This program is distributed in the hope that it will be useful, but WITHOUT ANY
% WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
% PARTICULAR PURPOSE.  See the GNU General Public License for more details.

% You should have received a copy of the GNU General Public License along with this
% program; if not, see <http://www.gnu.org/licenses/>.

%
% CompositeLikelihoodCalculator.m
%
classdef CompositeLikelihoodCalculator < LikelihoodCalculator

  properties (SetAccess = private)
    Calculators
  end

  methods (Access = public)

    function AddCalculator(this, calculator)
      this.Calculators{length(this.Calculators) + 1} = calculator;
    end

    function logLikelihood = LogLikelihood( ...
        this, ...
        stateTransitionFunction, ...
        observationFunction, ...
        stateTransitionNoise, ...
        observationNoise)
      for i = 1:length(this.Calculators)
        if nargin < 4
          logLh(i) = this.Calculators{i}.LogLikelihood( ...
              stateTransitionFunction, ...
              observationFunction);
        else
          logLh(i) = this.Calculators{i}.LogLikelihood( ...
              stateTransitionFunction, ...
              observationFunction, ...
              stateTransitionNoise, ...
              observationNoise);
        end
      end
      logLikelihood = sum(logLh);
    end

    function OnAccept(this, j, in, out, aspect)
      % TODO: in the current implementation we can't log multiple sequences when accepted.
      this.Calculators{1}.OnAccept(j, in, out, aspect);
    end

  end

end
