%%% Monte Carlo Dynamic Classifier tool - MCDC tool
% http://www.ismvideo.org/NDE0UP/MCDCtool/en/index.html

% Copyright (C) 2014-2015 Gareth W. Peters, Tomoko Matsui, Francois Septier, and Akira Tamamori

% This program is free software; you can redistribute it and/or modify it under the terms
% of the GNU General Public License as published by the Free Software Foundation; either
% version 2 of the License, or (at your option) any later version.

% This program is distributed in the hope that it will be useful, but WITHOUT ANY
% WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
% PARTICULAR PURPOSE.  See the GNU General Public License for more details.

% You should have received a copy of the GNU General Public License along with this
% program; if not, see <http://www.gnu.org/licenses/>.

%
% GeneralizedParetoDistribution.m
%
classdef GeneralizedParetoDistribution < ContinuousUnivariateDistribution

  properties (SetAccess = private)
    K
    Sigma
    Theta
  end

  methods (Access = public)

    function this = GeneralizedParetoDistribution(k, sigma, theta, dim)
      if nargin < 4
        dim = 1;
      end
      this = this@ContinuousUnivariateDistribution( ...
          makedist('GeneralizedPareto', 'k', k, 'sigma', sigma, 'theta', theta), ...
          dim);
      this.K = k;
      this.Sigma = sigma;
      this.Theta = theta;
    end

    function result = LogPdf(this, x)
      k = this.K;
      s = this.Sigma;
      t = this.Theta;
      if k == 0
        result = -this.dim * log(s) - sum((x - t) / s, 2);
      else
        result = -this.dim * log(s) - (1 + 1 / k) * sum(log(1 + k * (x - t) / s), 2);
      end
    end

  end

end
