%%% Monte Carlo Dynamic Classifier tool - MCDC tool
% http://www.ismvideo.org/NDE0UP/MCDCtool/en/index.html

% Copyright (C) 2014-2015 Gareth W. Peters, Tomoko Matsui, Francois Septier, and Akira Tamamori

% This program is free software; you can redistribute it and/or modify it under the terms
% of the GNU General Public License as published by the Free Software Foundation; either
% version 2 of the License, or (at your option) any later version.

% This program is distributed in the hope that it will be useful, but WITHOUT ANY
% WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
% PARTICULAR PURPOSE.  See the GNU General Public License for more details.

% You should have received a copy of the GNU General Public License along with this
% program; if not, see <http://www.gnu.org/licenses/>.

%
% MCDCStrategyChoice3.m
%

classdef MCDCStrategyChoice3 < MCDCStrategy

  properties (Access = private)
    stateSigma
    obsSigma
  end

  methods (Static, Access = public)

    function result = Prepare(stateSigma, obsSigma)
      result = @(algorithm) MCDCStrategyChoice3(algorithm, stateSigma, obsSigma);
    end

  end

  methods (Access = public)

    function this = MCDCStrategyChoice3(algorithm, stateSigma, obsSigma)
      this = this@MCDCStrategy(algorithm);
      this.stateSigma = stateSigma;
      this.obsSigma = obsSigma;
    end

    function result = StateMean(this, j, in, out)
      if j == 1
        this.SetInitialStateMean(in);
        result = this.initialStateMean;
      else
        result = reshape(out.FVAt(j - 1), size(this.initialStateMean));
      end
    end

    function result = ObsMean(this, j, in, out)
      if j == 1
        this.SetInitialObsMean(in);
        result = this.initialObsMean;
      else
        result = reshape(out.GVAt(j - 1), size(this.initialObsMean));
      end
    end

    function result = StateSigma(this, j, gramMatrix)
      result = this.stateSigma;
    end

    function result = ObsSigma(this, j, gramMatrix)
      result = this.obsSigma;
    end

    function result = AcceptanceLogProbability(this, j, in, out, logLikelihood, stateModel, obsModel)
      if j == 1
        result = 0;
      else
        result = this.LikelihoodContribution(j, out, logLikelihood) ...
               + this.PriorContribution(j, in, out, stateModel, obsModel);
      end
    end

  end

end
