%%% Monte Carlo Dynamic Classifier tool - MCDC tool
% http://www.ismvideo.org/NDE0UP/MCDCtool/en/index.html

% Copyright (C) 2014-2015 Gareth W. Peters, Tomoko Matsui, Francois Septier, and Akira Tamamori

% This program is free software; you can redistribute it and/or modify it under the terms
% of the GNU General Public License as published by the Free Software Foundation; either
% version 2 of the License, or (at your option) any later version.

% This program is distributed in the hope that it will be useful, but WITHOUT ANY
% WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
% PARTICULAR PURPOSE.  See the GNU General Public License for more details.

% You should have received a copy of the GNU General Public License along with this
% program; if not, see <http://www.gnu.org/licenses/>.

%
% NormalDistribution.m
%
classdef NormalDistribution < ContinuousUnivariateDistribution

  properties (SetAccess = private)
    Mean
    Variance
  end

  methods (Access = public)

    function this = NormalDistribution(mean, variance, dim)
      if nargin < 3
        dim = 1;
      end
      this = this@ContinuousUnivariateDistribution( ...
          makedist('Normal', 'mu', mean, 'sigma', sqrt(variance)), ...
          dim);
      this.Mean = mean;
      this.Variance = variance;
    end

    function result = LogPdf(this, x)
      m = this.Mean;
      v = this.Variance;
      result = -0.5 * (this.dim * log(2 * pi * v) + sum((x - m) .^ 2, 2) / v);
    end

  end

end
