%%% Monte Carlo Dynamic Classifier tool - MCDC tool
% http://www.ismvideo.org/NDE0UP/MCDCtool/en/index.html

% Copyright (C) 2014-2015 Gareth W. Peters, Tomoko Matsui, Francois Septier, and Akira Tamamori

% This program is free software; you can redistribute it and/or modify it under the terms
% of the GNU General Public License as published by the Free Software Foundation; either
% version 2 of the License, or (at your option) any later version.

% This program is distributed in the hope that it will be useful, but WITHOUT ANY
% WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
% PARTICULAR PURPOSE.  See the GNU General Public License for more details.

% You should have received a copy of the GNU General Public License along with this
% program; if not, see <http://www.gnu.org/licenses/>.

%
% PMCMC2.m
%
% input:
%   xGrids: grid data for state variable (cell matrix for each dimention)
%   initialStateKernelGens: cell matrix of objects to generate initial kernel function for each dimension of state variable (cell matrix of 'KernelGenerator')
%   initialObsKernelGens: cell matrix of objects to generate initial kernel function for each dimension of observation variable (cell matrix of 'KernelGenerator')
%   initialStateMeanFuncs: cell matrix of initial mean function of state variable for each dimention
%   initialObsMeanFuncs: cell matrix of initial mean function of observation variable for each dimention
%   gridDimForGramMatrix: dimension to generate kernel covariance matrix (used only for PMCMC2)
%   splineHandle: function handler to generate spline interpolation function
%   x0: initial value for state variable
%   xaux: auxiliary variable for state variable
%   u: control input for state variable
%   z: time series data of observation values
%   N: number of particles for particle filter
%   J: iteratrion for MCMC step
%   K: time difference between state and observation
%   aspect: Aspect object
%
% output:
%   SKP: parameters for kernel function of state variable through the iterations
%   OKP: parameters for kernel function of observation variable through the iterations
%   FV: state values on each grid through the iteration
%   GV: observation values on each grid through the iteration
%   XE: transition of state variable through the iteration
%   YE: transition of observation variable through the iteration
%   loglik: transition of log-likehood through the iteration
%

classdef PMCMC2 < Algorithm

  methods (Access = public)

    function this = PMCMC2(kernelGeneratorStrategy, mcdcStrategy)
      this = this@Algorithm( ...
          kernelGeneratorStrategy, ...
          mcdcStrategy);
    end

    function gramMatrixGrid = getGramMatrixGrid(this, in, gridData, aspect)
      aspect.Log('  Creating GramMatrix using %d dim...', in.gridDimForGramMatrix);
      gramMatrixGrid = in.xGrids{in.gridDimForGramMatrix};
    end

  end

  methods (Access = protected)

    function indices = getIndicesFromGridData(this, gridData, pivot)
      indices = gridData.GetIndices(pivot);
    end

  end

end
