%%% Monte Carlo Dynamic Classifier tool - MCDC tool
% http://www.ismvideo.org/NDE0UP/MCDCtool/en/index.html

% Copyright (C) 2014-2015 Gareth W. Peters, Tomoko Matsui, Francois Septier, and Akira Tamamori

% This program is free software; you can redistribute it and/or modify it under the terms
% of the GNU General Public License as published by the Free Software Foundation; either
% version 2 of the License, or (at your option) any later version.

% This program is distributed in the hope that it will be useful, but WITHOUT ANY
% WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
% PARTICULAR PURPOSE.  See the GNU General Public License for more details.

% You should have received a copy of the GNU General Public License along with this
% program; if not, see <http://www.gnu.org/licenses/>.

%
% KitagawaModelPSMCEstimation.m
%

rng('default');

%
% Generate state and observation time series from Kitagawa model
%

T  = 1000;
a1 = 0.5;
b1 = 28;
c1 = 8;
a2 = 0.6;
b2 = 30;
c2 = 10;
d1 = 0.05;
d2 = 0.06;
d3 = 0.07;
d4 = 0.08;
q  = 1e-1;
r  = 1e-1;

[x, y] = KitagawaModel(T, a1, b1, c1, a2, b2, c2, d1, d2, d3, d4, q, r);

save('logs/KitagawaModelPSMC_Data.mat');

grids = { ...
  [-30:2:30], ...
  [-30:2:30]  ...
};

stateKernelGens = { ...
  RBFKernelGenerator(FixedValueDistribution(2.500395), FixedValueDistribution(0.436103)), ...
  RBFKernelGenerator(FixedValueDistribution(0.092902), FixedValueDistribution(3.818451))  ...
};

obsKernelGens = {
  RBFKernelGenerator(FixedValueDistribution(6.892343), FixedValueDistribution(0.281778)), ...
  RBFKernelGenerator(FixedValueDistribution(8.082916), FixedValueDistribution(3.137134)), ...
  RBFKernelGenerator(FixedValueDistribution(7.977223), FixedValueDistribution(8.766439))  ...
};

stateMeanFuncs = { ...
  @(x) (repmat(0, 1, size(x, 2))), ...
  @(x) (repmat(0, 1, size(x, 2)))  ...
};

obsMeanFuncs = { ...
  @(x) (repmat(0, 1, size(x, 2))), ...
  @(x) (repmat(0, 1, size(x, 2))), ...
  @(x) (repmat(0, 1, size(x, 2)))  ...
};

J = 2000;

aspect = Aspect();
aspect.LogFileName = 'logs/KitagawaModelPSMC.log';
aspect.MatFileNamePrefix = 'logs/KitagawaModelPSMC';
aspect.SavesIntermediateMat = true;
aspect.IntermediateMatInterval = 100;

disp('PSMC Estimation using SPAPI');

stateSplineHandle = @GenericSpline;
obsSplineHandle = @GenericSpline;
algorithm = PSMC();
algorithm.SetPMCMCProbability(1.0); % Don't use MH-Gibbs

stateModel = GaussianProcessModel(ModelKind.State, algorithm);
% stateModel = FixedModel(ModelKind.State, VectorValuedFunction(stateMeanFuncs));
algorithm.SetStateModel(stateModel);

obsModel = GaussianProcessModel(ModelKind.Observation, algorithm);
% obsModel = FixedModel(ModelKind.Observation, VectorValuedFunction(obsMeanFuncs));
algorithm.SetObsModel(obsModel);

x0 = zeros(1, 2);
xaux = [];
u = [];
q = 1;
r = 0.1;
N = 1000;
likelihoodCalculator = ParticleFilter(x0, xaux, u, y, q, r, N, 0);
algorithm.SetLikelihoodCalculator(likelihoodCalculator);

[ IDX, SKP, OKP, FV, GV, XE, YE, loglik ] = MCDCTrain( ...
  algorithm, ...
  grids, ...
  stateKernelGens, ...
  obsKernelGens, ...
  stateMeanFuncs, ...
  obsMeanFuncs, ...
  0, ... % Not used in PSMC estimation
  stateSplineHandle, ...
  obsSplineHandle, ...
  x0, ...
  [], ... % No auxiliary states
  [], ... % No controls
  y', ...
  N, ...
  J, ...
  0, ...
  aspect ...
);

save('logs/KitagawaModelPSMC.mat');

disp('Done');
