%%% Monte Carlo Dynamic Classifier tool - MCDC tool
% http://www.ismvideo.org/NDE0UP/MCDCtool/en/index.html

% Copyright (C) 2014-2015 Gareth W. Peters, Tomoko Matsui, Francois Septier, and Akira Tamamori

% This program is free software; you can redistribute it and/or modify it under the terms
% of the GNU General Public License as published by the Free Software Foundation; either
% version 2 of the License, or (at your option) any later version.

% This program is distributed in the hope that it will be useful, but WITHOUT ANY
% WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
% PARTICULAR PURPOSE.  See the GNU General Public License for more details.

% You should have received a copy of the GNU General Public License along with this
% program; if not, see <http://www.gnu.org/licenses/>.

%
% LinearStateSpaceModelPMCMC2Estimation.m
%

rng('default');

%
% Generate state and observation time series from Linear state space model
%

T    = 1000;
k1   = 0.1;
k2   = 0.1;
mass = 1.0;
q    = 0.1;
r    = 0.1;
dt   = 1.0;

[x, y, F, H] = LinearStateSpaceModel(T, k1, k2, mass, q, r, dt);

save('logs/LinearStateSpaceModelPMCMC2_Data.mat');

grids = { ...
  [-2:0.1:2], ...
  [-2:0.1:2]  ...
};

stateKernelGens = { ...
  RBFKernelGenerator(UniformDistribution(0.01, 10), UniformDistribution(0.01, 10)), ...
  RBFKernelGenerator(UniformDistribution(0.01, 10), UniformDistribution(0.01, 10))  ...
};

obsKernelGens = {
  RBFKernelGenerator(UniformDistribution(0.01, 10), UniformDistribution(0.01, 10))  ...
};

stateMeanFuncs = { ...
  @(x) x(1,:), ...
  @(x) x(2,:)  ...
};

obsMeanFuncs = { ...
  @(x) x(1,:)  ...
};

J  = 100000;

gridDimForGramMatrix = 1; % Use x1 for building gram matrix.

aspect = Aspect();
aspect.LogFileName = 'logs/LinearStateSpaceModelPMCMC2.log';
aspect.MatFileNamePrefix = 'logs/LinearStateSpaceModelPMCMC2';
aspect.SavesIntermediateMat = true;

disp('PMCMC2 Estimation using SPAPI');

%%% Choice 1
% kernelGeneratorStrategy = @RBFKernelGeneratorStrategyChoice1;

%%% Choice 2
kernelGeneratorStrategy = @RBFKernelGeneratorStrategyChoice2;

%%% Choice 1
% mcdcSrategy = @MCDCStrategyChoice1;

%%% Choice 2
mcdcStrategy = @MCDCStrategyChoice2;

%%% Choice 3: build gram matrices here.
% for i = 1:size(stateMeanFuncs, 2)
%   stateSigma{i} = eye(length(grids{gridDimForGramMatrix}));
% end
% for i = 1:size(obsMeanFuncs, 2)
%   obsSigma{i} = eye(length(grids{gridDimForGramMatrix}));
% end
% mcdcStrategy = MCDCStrategyChoice3.Prepare(stateSigma, obsSigma);

stateSplineHandle = @GenericSpline;
obsSplineHandle = @GenericSpline;
algorithm = PMCMC2(kernelGeneratorStrategy, mcdcStrategy);
algorithm.SetPMCMCProbability(1.0); % Don't use MH-Gibbs

stateModel = GaussianProcessModel(ModelKind.State, algorithm);
% stateModel = FixedModel(ModelKind.State, VectorValuedFunction(stateMeanFuncs));
algorithm.SetStateModel(stateModel);

obsModel = GaussianProcessModel(ModelKind.Observation, algorithm);
% obsModel = FixedModel(ModelKind.Observation, VectorValuedFunction(obsMeanFuncs));
algorithm.SetObsModel(obsModel);

x0 = zeros(1, 2);
xaux = [];
u = [];
q = 1;
r = 0.1;
N = 500;
likelihoodCalculator = PMCMCParticleFilter(x0, xaux, u, y, q, r, N, 0);
algorithm.SetLikelihoodCalculator(likelihoodCalculator);

[ IDX, SKP, OKP, FV, GV, XE, YE, loglik ] = MCDCTrain( ...
  algorithm, ...
  grids, ...
  stateKernelGens, ...
  obsKernelGens, ...
  stateMeanFuncs, ...
  obsMeanFuncs, ...
  gridDimForGramMatrix, ...
  stateSplineHandle, ...
  obsSplineHandle, ...
  x0, ...
  [], ... % No auxiliary states
  [], ... % No controls
  y', ...
  N, ...
  J, ...
  0, ...
  aspect ...
);

save('logs/LinearStateSpaceModelPMCMC2.mat');

disp('Done');
