%%% Monte Carlo Dynamic Classifier tool - MCDC tool
% http://www.ismvideo.org/NDE0UP/MCDCtool/en/index.html

% Copyright (C) 2014-2015 Gareth W. Peters, Tomoko Matsui, Francois Septier, and Akira Tamamori

% This program is free software; you can redistribute it and/or modify it under the terms
% of the GNU General Public License as published by the Free Software Foundation; either
% version 2 of the License, or (at your option) any later version.

% This program is distributed in the hope that it will be useful, but WITHOUT ANY
% WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
% PARTICULAR PURPOSE.  See the GNU General Public License for more details.

% You should have received a copy of the GNU General Public License along with this
% program; if not, see <http://www.gnu.org/licenses/>.

%
% LorenzModelPSMCEstimation.m
%

rng('default');

%
% Generate state and observation time series from Lorenz model
%

T      = 1000;
Sigma  = 10;
Rho    = 28;
Beta   = 8/3;
Alpha0 = 1;
q      = 1e-1;
r      = 1e-1;
dt     = 1e-2;

[x, y] = LorenzModel(T, Sigma, Rho, Beta, Alpha0, q, r, dt);

save('logs/LorenzModelPSMC_Data.mat');

grids = { ...
  [-30:2:30], ...
  [-30:2:30]  ...
};

stateKernelGens = { ...
  RBFKernelGenerator(UniformDistribution(0.01, 10), UniformDistribution(0.01, 10)), ...
  RBFKernelGenerator(UniformDistribution(0.01, 10), UniformDistribution(0.01, 10))  ...
};

obsKernelGens = {
  RBFKernelGenerator(UniformDistribution(0.01, 10), UniformDistribution(0.01, 10)), ...
  RBFKernelGenerator(UniformDistribution(0.01, 10), UniformDistribution(0.01, 10)), ...
  RBFKernelGenerator(UniformDistribution(0.01, 10), UniformDistribution(0.01, 10))  ...
};

stateMeanFuncs = { ...
  @(x) x(1,:), ...
  @(x) x(2,:)  ...
};

obsMeanFuncs = { ...
  @(x) (repmat(0, 1, size(x, 2))), ...
  @(x) (repmat(0, 1, size(x, 2))), ...
  @(x) (repmat(0, 1, size(x, 2)))  ...
};

J  = 2000;

aspect = Aspect();
aspect.LogFileName = 'logs/LorenzModelPSMC.log';
aspect.MatFileNamePrefix = 'logs/LorenzModelPSMC';
aspect.SavesIntermediateMat = true;
aspect.IntermediateMatInterval = 100;

disp('PSMC Estimation using SPAPI');

stateSplineHandle = @GenericSpline;
obsSplineHandle = @GenericSpline;
algorithm = PSMC();
algorithm.SetPMCMCProbability(1.0); % Don't use MH-Gibbs

stateModel = GaussianProcessModel(ModelKind.State, algorithm);
% stateModel = FixedModel(ModelKind.State, VectorValuedFunction(stateMeanFuncs));
algorithm.SetStateModel(stateModel);

obsModel = GaussianProcessModel(ModelKind.Observation, algorithm);
% obsModel = FixedModel(ModelKind.Observation, VectorValuedFunction(obsMeanFuncs));
algorithm.SetObsModel(obsModel);

x0 = zeros(1, 2);
xaux = [];
u = [];
q = 1;
r = 0.1;
N = 500;
likelihoodCalculator = ParticleFilter(x0, xaux, u, y, q, r, N, 0);
algorithm.SetLikelihoodCalculator(likelihoodCalculator);

[ IDX, SKP, OKP, FV, GV, XE, YE, loglik ] = MCDCTrain( ...
  algorithm, ...
  grids, ...
  stateKernelGens, ...
  obsKernelGens, ...
  stateMeanFuncs, ...
  obsMeanFuncs, ...
  0, ... % Not used in PSMC estimation
  stateSplineHandle, ...
  obsSplineHandle, ...
  x0, ...
  [], ... % No auxiliary states
  [], ... % No controls
  y', ...
  N, ...
  J, ...
  0, ...
  aspect ...
);

save('logs/LorenzModelPSMC.mat');

disp('Done');
