%%% Monte Carlo Dynamic Classifier tool - MCDC tool
% http://www.ismvideo.org/NDE0UP/MCDCtool/en/index.html

% Copyright (C) 2014 Gareth W. Peters, Tomoko Matsui, Francois Septier, and Akira Tamamori

% This program is free software; you can redistribute it and/or modify it under the terms
% of the GNU General Public License as published by the Free Software Foundation; either
% version 2 of the License, or (at your option) any later version.

% This program is distributed in the hope that it will be useful, but WITHOUT ANY
% WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
% PARTICULAR PURPOSE.  See the GNU General Public License for more details.

% You should have received a copy of the GNU General Public License along with this
% program; if not, see <http://www.gnu.org/licenses/>.

%
% Cycle.m
%
% see below:
%
%   C. Andrieu, A. Doucet, and R. Holenstein.  Particle Markov chain Monte Carlo
%   methods.  J. R. Statist. Soc. B (2010) 72, Part 3, pp. 269-342.
%
%   Formulas (14) and (15) in section 3.1.
%

classdef Cycle < handle

  properties
    sv
    sw
    coefs
  end

  methods

    function this = Cycle()
      this.sv = sqrt(0.1);
      this.sw = sqrt(0.1);
      this.coefs = [0.1];
    end

    function [X, Y, V, W] = generate(this, T)
      V = normrnd(0, this.sv, T, 1);
      W = normrnd(0, this.sw, T, 1);
      V(1) = 0;
      [X, Y] = this.generateFromNoise(V, W);
    end

    function [Y, W] = generateFromState(this, X)
      T = length(X);
      W = normrnd(0, this.sw, T, 1);
      for t = 1:T
        Y(t, 1) = this.observation(X(t, 1)) + W(t, 1);
      end
    end

    function [X, Y] = generateFromNoise(this, V, W)
      T = length(V);
      X = zeros(T, 1);
      Y = zeros(T, 1);
      t = 1;
      X(t, 1) = rand * 2 * pi;
      Y(t, 1) = this.observation(X(t, 1)) + W(t, 1);
      for t = 2:T
        X(t, 1) = this.transition(X(t - 1, 1)) + V(t, 1);
        Y(t, 1) = this.observation(X(t, 1)) + W(t, 1);
      end
    end

    function x = transition(this, xprev)
      x = mod(xprev + this.coefs(1), 2 * pi);
    end

    function y = observation(this, x)
      y = sin(x);
    end

  end

end
