%%% Monte Carlo Dynamic Classifier tool - MCDC tool
% http://www.ismvideo.org/NDE0UP/MCDCtool/en/index.html

% Copyright (C) 2014 Gareth W. Peters, Tomoko Matsui, Francois Septier, and Akira Tamamori

% This program is free software; you can redistribute it and/or modify it under the terms
% of the GNU General Public License as published by the Free Software Foundation; either
% version 2 of the License, or (at your option) any later version.

% This program is distributed in the hope that it will be useful, but WITHOUT ANY
% WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
% PARTICULAR PURPOSE.  See the GNU General Public License for more details.

% You should have received a copy of the GNU General Public License along with this
% program; if not, see <http://www.gnu.org/licenses/>.

%
% NonLinearSSM_PMCMC2_NonGaussianNoise.m
%

%%% Initialization

addpath('../');
rng('shuffle');

%%% Generate a sample model

% [X, Y, V, W] = NonLinearSSM().generate(100);
load('NonLinearSSM.mat');

%%% Setup the estimator

grids = { ...
  [-50:1:50] ...
};

stateKernelGens = { ...
  RBFKernelGenerator(UniformDistribution(0.01, 10), UniformDistribution(0.01, 10)) ...
};

stateMeanFuncs = { ...
  @(x) (x / 2 + 25 * x ./ (1 + x .^ 2)) ...
};

obsKernelGens = { ...
  RBFKernelGenerator(UniformDistribution(0.01, 10), UniformDistribution(0.01, 10)) ...
};

obsMeanFuncs = { ...
  @(x) (x .^ 2 / 20) ...
};

aspect = Aspect();
aspect.LogFileName = '../logs/NonLinearSSM_PMCMC2_NonGaussianNoise.log';
aspect.MatFileNamePrefix = '../logs/NonLinearSSM_PMCMC2_NonGaussianNoise';
aspect.SavesIntermediateMat = true;
aspect.IntermediateMatInterval = 1000;

J = 100; % MCMC iteration count
gridDimForGramMatrix = 1; % Use x1 for building gram matrix.

disp('PMCMC2 Estimation using SPAPI');

kernelGeneratorStrategy = @RBFKernelGeneratorStrategyChoice2;
mcdcStrategy = @MCDCStrategyChoice2;

stateSplineHandle = @SimpleSplineInGrid;
obsSplineHandle = @SimpleSpline;
algorithm = PMCMC2(kernelGeneratorStrategy, mcdcStrategy);

stateModel = GaussianProcessModel(ModelKind.State, algorithm);
algorithm.SetStateModel(stateModel);

obsModel = GaussianProcessModel(ModelKind.Observation, algorithm);
algorithm.SetObsModel(obsModel);

x0 = 0;
xaux = [];
u = [0; 8 * cos(1.2 * [2:100])'];
q = 1;
r = 0.1;
N = 1000;
likelihoodCalculator = PMCMCParticleFilter(x0, xaux, u, Y, q, r, N, 0);
algorithm.SetLikelihoodCalculator(likelihoodCalculator);

%%% Set noise distributions (default = NormalDistribution)

xdim = size(x0, 2);
stateNoise = CauchyDistribution(0, q, 1);
% stateNoise = NormalDistribution(0, q, xdim);
% stateNoise = TDistribution(1, xdim);
% stateNoise = TLocationScaleDistribution(0, q, 1, xdim);
% stateNoise = UniformDistribution(-100, 100, xdim);
algorithm.SetStateNoise(stateNoise);

ydim = size(Y, 2);
obsNoise = CauchyDistribution(0, r, ydim);
% obsNoise = NormalDistribution(0, r, ydim);
% obsNoise = TDistribution(1, ydim);
% obsNoise = TLocationScaleDistribution(0, r, 1, ydim);
% obsNoise = UniformDistribution(-100, 100, ydim);
algorithm.SetObsNoise(obsNoise);

%%% Run the estimator

[ IDX, SKP, OKP, FV, GV, XE, YE, loglik ] = MCDCTrain( ...
  algorithm, ...
  grids, ...
  stateKernelGens, ...
  obsKernelGens, ...
  stateMeanFuncs, ...
  obsMeanFuncs, ...
  gridDimForGramMatrix, ...
  stateSplineHandle, ...
  obsSplineHandle, ...
  x0, ...
  [], ... % No auxiliary states
  u', ... % No controls
  Y', ...
  N, ...
  J, ...
  0, ...
  aspect ...
);

save('../logs/NonLinearSSM_PMCMC2_NonGaussianNoise.mat');

disp('Done');
