%%% Monte Carlo Dynamic Classifier tool - MCDC tool
% http://www.ismvideo.org/NDE0UP/MCDCtool/en/index.html

% Copyright (C) 2014 Gareth W. Peters, Tomoko Matsui, Francois Septier, and Akira Tamamori

% This program is free software; you can redistribute it and/or modify it under the terms
% of the GNU General Public License as published by the Free Software Foundation; either
% version 2 of the License, or (at your option) any later version.

% This program is distributed in the hope that it will be useful, but WITHOUT ANY
% WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
% PARTICULAR PURPOSE.  See the GNU General Public License for more details.

% You should have received a copy of the GNU General Public License along with this
% program; if not, see <http://www.gnu.org/licenses/>.

%
% NonLinearSSM_PMCMC_FixedModel_State.m
%

%%% Initialization

addpath('../');
rng('shuffle');

%%% Generate a sample model

% [X, Y, V, W] = NonLinearSSM().generate(100);
load('NonLinearSSM.mat');

%%% Setup the estimator

grids = { ...
  [-50:1:50] ...
};

stateKernelGens = { ...
  RBFKernelGenerator(UniformDistribution(0.01, 10), UniformDistribution(0.01, 10)) ...
};

stateMeanFuncs = { ...
  @(x) (x / 2 + 25 * x ./ (1 + x .^ 2)) ...
};

obsKernelGens = { ...
  RBFKernelGenerator(UniformDistribution(0.01, 10), UniformDistribution(0.01, 10)) ...
};

obsMeanFuncs = { ...
  @(x) (x .^ 2 / 20) ...
};

aspect = Aspect();
aspect.LogFileName = '../logs/NonLinearSSM_PMCMC_FixedModel_State.log';
aspect.MatFileNamePrefix = '../logs/NonLinearSSM_PMCMC_FixedModel_State';
aspect.SavesIntermediateMat = true;
aspect.IntermediateMatInterval = 1000;

J = 100; % MCMC iteration count

disp('PMCMC Estimation using SPAPI');

stateSplineHandle = @SimpleSplineInGrid;
obsSplineHandle = @SimpleSpline;
algorithm = PMCMC();

%%% Set the state transition function if it is known

% stateModel = GaussianProcessModel(ModelKind.State, algorithm);
stateModel = FixedModel(ModelKind.State, VectorValuedFunction(stateMeanFuncs));
algorithm.SetStateModel(stateModel);

%%% Set the observation function if it is known

obsModel = GaussianProcessModel(ModelKind.Observation, algorithm);
% obsModel = FixedModel(ModelKind.Observation, VectorValuedFunction(obsMeanFuncs));
algorithm.SetObsModel(obsModel);

x0 = 0;
xaux = [];
u = [0; 8 * cos(1.2 * [2:100])'];
q = 1;
r = 0.1;
N = 1000;
likelihoodCalculator = PMCMCParticleFilter(x0, xaux, u, Y, q, r, N, 0);
algorithm.SetLikelihoodCalculator(likelihoodCalculator);

%%% Run the estimator

[ IDX, SKP, OKP, FV, GV, XE, YE, loglik ] = MCDCTrain( ...
  algorithm, ...
  grids, ...
  stateKernelGens, ...
  obsKernelGens, ...
  stateMeanFuncs, ...
  obsMeanFuncs, ...
  0, ... % Not used in PMCMC estimation
  stateSplineHandle, ...
  obsSplineHandle, ...
  x0, ...
  [], ... % No auxiliary states
  u', ... % No controls
  Y', ...
  N, ...
  J, ...
  0, ...
  aspect ...
);

save('../logs/NonLinearSSM_PMCMC_FixedModel_State.mat');

disp('Done');
