%%% Monte Carlo Dynamic Classifier tool - MCDC tool
% http://www.ismvideo.org/NDE0UP/MCDCtool/en/index.html

% Copyright (C) 2014 Gareth W. Peters, Tomoko Matsui, Francois Septier, and Akira Tamamori

% This program is free software; you can redistribute it and/or modify it under the terms
% of the GNU General Public License as published by the Free Software Foundation; either
% version 2 of the License, or (at your option) any later version.

% This program is distributed in the hope that it will be useful, but WITHOUT ANY
% WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
% PARTICULAR PURPOSE.  See the GNU General Public License for more details.

% You should have received a copy of the GNU General Public License along with this
% program; if not, see <http://www.gnu.org/licenses/>.

%
% NonLinearSSM_PMCMC_Noise_Laplace.m
%

%%% Initialization

addpath('../');
rng('shuffle');

%%% Generate a sample model

% laplace = LaplaceDistribution(0, 2);
% V = laplace.Random(100);
% W = laplace.Random(100);
% [X, Y] = NonLinearSSM().generateFromNoise(V, W);
% load('NonLinearSSM_LaplaceNoise.mat');
load('NonLinearSSM.mat');

%%% Setup the estimator

grids = { ...
  [-50:1:50] ...
};

stateKernelGens = { ...
  RBFKernelGenerator(UniformDistribution(0.01, 10), UniformDistribution(0.01, 10)) ...
};

stateMeanFuncs = { ...
  @(x) (x / 2 + 25 * x ./ (1 + x .^ 2)) ...
};

obsKernelGens = { ...
  RBFKernelGenerator(UniformDistribution(0.01, 10), UniformDistribution(0.01, 10)) ...
};

obsMeanFuncs = { ...
  @(x) (x .^ 2 / 20) ...
};

aspect = Aspect();
aspect.LogFileName = '../logs/NonLinearSSM_PMCMC_Noise_Laplace.log';
aspect.MatFileNamePrefix = '../logs/NonLinearSSM_PMCMC_Noise_Laplace';
aspect.SavesIntermediateMat = true;
aspect.IntermediateMatInterval = 1000;

J = 100; % MCMC iteration count

disp('PMCMC Estimation using SPAPI');

stateSplineHandle = @SimpleSplineInGrid;
obsSplineHandle = @SimpleSpline;
algorithm = PMCMC();

stateModel = GaussianProcessModel(ModelKind.State, algorithm);
algorithm.SetStateModel(stateModel);

obsModel = GaussianProcessModel(ModelKind.Observation, algorithm);
algorithm.SetObsModel(obsModel);

x0 = 0;
xaux = [];
u = [0; 8 * cos(1.2 * [2:100])'];
q = 0; % Not used as we set noise distributions explicitly below.
r = 0; % Not used as we set noise distributions explicitly below.
N = 1000;
likelihoodCalculator = PMCMCParticleFilter(x0, xaux, u, Y, q, r, N, 0);
algorithm.SetLikelihoodCalculator(likelihoodCalculator);

%%% Set noise distributions (default = NormalDistribution)

% stateNoise = NormalDistribution(0, 1, 1);
stateNoise = LaplaceDistribution(0, 1);
algorithm.SetStateNoise(stateNoise);

% obsNoise = NormalDistribution(0, 0.1, 1);
obsNoise = LaplaceDistribution(0, 0.1);
algorithm.SetObsNoise(obsNoise);

%%% Run the estimator

[ IDX, SKP, OKP, FV, GV, XE, YE, loglik ] = MCDCTrain( ...
  algorithm, ...
  grids, ...
  stateKernelGens, ...
  obsKernelGens, ...
  stateMeanFuncs, ...
  obsMeanFuncs, ...
  0, ... % Not used in PMCMC estimation
  stateSplineHandle, ...
  obsSplineHandle, ...
  x0, ...
  [], ... % No auxiliary states
  u', ... % No controls
  Y', ...
  N, ...
  J, ...
  0, ...
  aspect ...
);

save('../logs/NonLinearSSM_PMCMC_Noise_Laplace.mat');

disp('Done');
