%%% Monte Carlo Dynamic Classifier tool - MCDC tool
% http://www.ismvideo.org/NDE0UP/MCDCtool/en/index.html

% Copyright (C) 2014 Gareth W. Peters, Tomoko Matsui, Francois Septier, and Akira Tamamori

% This program is free software; you can redistribute it and/or modify it under the terms
% of the GNU General Public License as published by the Free Software Foundation; either
% version 2 of the License, or (at your option) any later version.

% This program is distributed in the hope that it will be useful, but WITHOUT ANY
% WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
% PARTICULAR PURPOSE.  See the GNU General Public License for more details.

% You should have received a copy of the GNU General Public License along with this
% program; if not, see <http://www.gnu.org/licenses/>.

%
% NonLinearSSM_Test.m
%

%%% Initialization

addpath('../');
rng('shuffle');

%%% Generate two sample sequences

generator1 = NonLinearSSM();
[X1, Y1, V1, W1] = generator1.generate(100);

generator2 = NonLinearSSM();
generator2.sw = sqrt(1);
generator2.coefs(1) = 20;
generator2.coefs(4) = 15;
[X2, Y2, V2, W2] = generator2.generate(100);

%%% Setup the estimator 1

grids = { ...
  [-50:1:50] ...
};

stateKernelGens = { ...
  RBFKernelGenerator(UniformDistribution(0.01, 10), UniformDistribution(0.01, 10)) ...
};

stateMeanFuncs = { ...
  @(x) (x / 2 + 25 * x ./ (1 + x .^ 2)) ...
};

obsKernelGens = { ...
  RBFKernelGenerator(UniformDistribution(0.01, 10), UniformDistribution(0.01, 10)) ...
};

obsMeanFuncs = { ...
  @(x) (x .^ 2 / 20) ...
};

J = 100; % MCMC iteration count
gridDimForGramMatrix = 1; % Use x1 for building gram matrix.

disp('PMCMC2 Estimation using SPAPI');

kernelGeneratorStrategy = @RBFKernelGeneratorStrategyChoice2;
mcdcStrategy = @MCDCStrategyChoice2;

stateSplineHandle = @SimpleSplineInGrid;
obsSplineHandle = @SimpleSpline;
algorithm = PMCMC2(kernelGeneratorStrategy, mcdcStrategy);

stateModel = GaussianProcessModel(ModelKind.State, algorithm);
algorithm.SetStateModel(stateModel);

obsModel = GaussianProcessModel(ModelKind.Observation, algorithm);
algorithm.SetObsModel(obsModel);

x0 = 0;
xaux = [];
u = [0; 8 * cos(1.2 * [2:100])'];
q = 1;
r = 0.1;
N = 1000;

%%% Run the estimator 1

likelihoodCalculator = PMCMCParticleFilter(x0, xaux, u, Y1, q, r, N, 0);
algorithm.SetLikelihoodCalculator(likelihoodCalculator);

aspect = Aspect();
aspect.LogFileName = '../logs/NonLinearSSM_Test1.log';
aspect.MatFileNamePrefix = '../logs/NonLinearSSM_Test1';
aspect.SavesIntermediateMat = true;
aspect.IntermediateMatInterval = 1000;

[ IDX1, SKP1, OKP1, FV1, GV1, XE1, YE1, loglik1 ] = MCDCTrain( ...
  algorithm, ...
  grids, ...
  stateKernelGens, ...
  obsKernelGens, ...
  stateMeanFuncs, ...
  obsMeanFuncs, ...
  gridDimForGramMatrix, ...
  stateSplineHandle, ...
  obsSplineHandle, ...
  x0, ...
  [], ... % No auxiliary states
  u', ... % No controls
  Y1', ...
  N, ...
  J, ...
  0, ...
  aspect ...
);

%%% Setup the estimator 2

grids = { ...
  [-50:1:50] ...
};

stateKernelGens = { ...
  RBFKernelGenerator(UniformDistribution(0.01, 10), UniformDistribution(0.01, 10)) ...
};

stateMeanFuncs = { ...
  @(x) (x / 2 + 20 * x ./ (1 + x .^ 2)) ...
};

obsKernelGens = { ...
  RBFKernelGenerator(UniformDistribution(0.01, 10), UniformDistribution(0.01, 10)) ...
};

obsMeanFuncs = { ...
  @(x) (x .^ 2 / 15) ...
};

J = 100; % MCMC iteration count
gridDimForGramMatrix = 1; % Use x1 for building gram matrix.

disp('PMCMC2 Estimation using SPAPI');

kernelGeneratorStrategy = @RBFKernelGeneratorStrategyChoice2;
mcdcStrategy = @MCDCStrategyChoice2;

stateSplineHandle = @SimpleSplineInGrid;
obsSplineHandle = @SimpleSpline;
algorithm = PMCMC2(kernelGeneratorStrategy, mcdcStrategy);

stateModel = GaussianProcessModel(ModelKind.State, algorithm);
algorithm.SetStateModel(stateModel);

obsModel = GaussianProcessModel(ModelKind.Observation, algorithm);
algorithm.SetObsModel(obsModel);

x0 = 0;
xaux = [];
u = [0; 8 * cos(1.2 * [2:100])'];
q = 1;
r = 0.1;
N = 1000;

%%% Run the estimator 2

likelihoodCalculator = PMCMCParticleFilter(x0, xaux, u, Y2, q, r, N, 0);
algorithm.SetLikelihoodCalculator(likelihoodCalculator);

aspect = Aspect();
aspect.LogFileName = '../logs/NonLinearSSM_Test2.log';
aspect.MatFileNamePrefix = '../logs/NonLinearSSM_Test2';
aspect.SavesIntermediateMat = true;
aspect.IntermediateMatInterval = 1000;

[ IDX2, SKP2, OKP2, FV2, GV2, XE2, YE2, loglik2 ] = MCDCTrain( ...
  algorithm, ...
  grids, ...
  stateKernelGens, ...
  obsKernelGens, ...
  stateMeanFuncs, ...
  obsMeanFuncs, ...
  gridDimForGramMatrix, ...
  stateSplineHandle, ...
  obsSplineHandle, ...
  x0, ...
  [], ... % No auxiliary states
  u', ... % No controls
  Y2', ...
  N, ...
  J, ...
  0, ...
  aspect ...
);

disp('Training done');

%%% Generate sequences for test

for i = 1:50
  [XT, YT, VT, WT] = generator1.generate(100);
  YTest(:, i * 2 - 1) = YT;
  [XT, YT, VT, WT] = generator2.generate(100);
  YTest(:, i * 2) = YT;
end

%%% Classification

u = [0; 8 * cos(1.2 * [2:100])'];
N = 1000;
modelFile1 = '../logs/NonLinearSSM_Test1_100.mat';
modelFile2 = '../logs/NonLinearSSM_Test2_100.mat';

for i = 1:100
  fprintf('Classification %d: ', i);
  L1(i) = MCDCTest(u', YTest(:, i)', N, modelFile1);
  L2(i) = MCDCTest(u', YTest(:, i)', N, modelFile2);
  Result(i) = L1(i) - L2(i);
  fprintf('Result = %d\n', Result(i));
end

save('../logs/NonLinearSSM_Test.mat');
