%%% Monte Carlo Dynamic Classifier tool - MCDC tool

% Copyright (C) 2014 Gareth W. Peters, Tomoko Matsui, Francois Septier, and Akira Tamamori

% This program is free software; you can redistribute it and/or modify it under the terms
% of the GNU General Public License as published by the Free Software Foundation; either
% version 2 of the License, or (at your option) any later version.

% This program is distributed in the hope that it will be useful, but WITHOUT ANY
% WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
% PARTICULAR PURPOSE.  See the GNU General Public License for more details.

% You should have received a copy of the GNU General Public License along with this
% program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
% Floor, Boston, MA 02110-1301 USA.

%
% BoundedNormalDistributionTest.m
%
classdef BoundedNormalDistributionTest < matlab.unittest.TestCase

  methods (Test)

    function testBoundedNormalDistribution(testCase)
      dist = BoundedNormalDistribution(2, 3, -0.5, 3.5);
      testCase.verifyEqual(dist.SourceDistribution.Mean, 2);
      testCase.verifyEqual(dist.SourceDistribution.Variance, 3);
      testCase.verifyEqual(dist.LowerBound, -0.5);
      testCase.verifyEqual(dist.UpperBound, 3.5);
    end

    function testPdf(testCase)
      dist = BoundedNormalDistribution(2, 3, -0.5, 3.5);
      actual = dist.Pdf(0);
      denom = normcdf(3.5, 2, sqrt(3)) - normcdf(-0.5, 2, sqrt(3));
      expected = normpdf(0, 2, sqrt(3)) / denom;
      testCase.verifyEqual(actual, expected, 'RelTol', 1e-6);
    end

    function testPdfLowerBound(testCase)
      dist = BoundedNormalDistribution(2, 3, -0.5, 3.5);
      actual = dist.Pdf(-1);
      expected = 0;
      testCase.verifyEqual(actual, expected, 'RelTol', 1e-6);
    end

    function testPdfUpperBound(testCase)
      dist = BoundedNormalDistribution(2, 3, -0.5, 3.5);
      actual = dist.Pdf(5);
      expected = 0;
      testCase.verifyEqual(actual, expected, 'RelTol', 1e-6);
    end

    function testPdfMultiVariate(testCase)
      dist = BoundedNormalDistribution(2, 3, -0.5, 3.5, 3);
      x = [0 0 0; 1 2 3; 5 4 3];
      actual = dist.Pdf(x);
      denom = normcdf(3.5, 2, sqrt(3)) - normcdf(-0.5, 2, sqrt(3));
      expected = [prod(normpdf(x(1:2, :), 2, sqrt(3)) / denom, 2); 0];
      testCase.verifyEqual(actual, expected, 'RelTol', 1e-6);
    end

    function testLogPdf(testCase)
      dist = BoundedNormalDistribution(2, 3, -0.5, 3.5);
      actual = dist.LogPdf(0);
      denom = normcdf(3.5, 2, sqrt(3)) - normcdf(-0.5, 2, sqrt(3));
      expected = log(normpdf(0, 2, sqrt(3)) / denom);
      testCase.verifyEqual(actual, expected, 'RelTol', 1e-6);
    end

    function testLogPdfLowerBound(testCase)
      dist = BoundedNormalDistribution(2, 3, -0.5, 3.5);
      actual = dist.LogPdf(-1);
      expected = -Inf;
      testCase.verifyEqual(actual, expected, 'RelTol', 1e-6);
    end

    function testLogPdfUpperBound(testCase)
      dist = BoundedNormalDistribution(2, 3, -0.5, 3.5);
      actual = dist.LogPdf(5);
      expected = -Inf;
      testCase.verifyEqual(actual, expected, 'RelTol', 1e-6);
    end

    function testLogPdfMultiVariate(testCase)
      dist = BoundedNormalDistribution(2, 3, -0.5, 3.5, 3);
      x = [0 0 0; 1 2 3; 5 4 3];
      actual = dist.LogPdf(x);
      denom = normcdf(3.5, 2, sqrt(3)) - normcdf(-0.5, 2, sqrt(3));
      expected = [log(prod(normpdf(x(1:2, :), 2, sqrt(3)) / denom, 2)); -Inf];
      testCase.verifyEqual(actual, expected, 'RelTol', 1e-6);
    end

  end

end
