%%% Monte Carlo Dynamic Classifier tool - MCDC tool
% http://www.ismvideo.org/NDE0UP/MCDCtool/en/index.html

% Copyright (C) 2014 Gareth W. Peters, Tomoko Matsui, Francois Septier, and Akira Tamamori

% This program is free software; you can redistribute it and/or modify it under the terms
% of the GNU General Public License as published by the Free Software Foundation; either
% version 2 of the License, or (at your option) any later version.

% This program is distributed in the hope that it will be useful, but WITHOUT ANY
% WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
% PARTICULAR PURPOSE.  See the GNU General Public License for more details.

% You should have received a copy of the GNU General Public License along with this
% program; if not, see <http://www.gnu.org/licenses/>.

%
% ExponentialDistributionTest.m
%
classdef ExponentialDistributionTest < matlab.unittest.TestCase

  methods (Test)

    function testExponentialDistribution(testCase)
      dist = ExponentialDistribution(2);
      testCase.verifyEqual(dist.Mu, 2);
    end

    function testPdf(testCase)
      dist = ExponentialDistribution(2);
      actual = dist.Pdf(1.35);
      expected = exppdf(1.35, 2);
      testCase.verifyEqual(actual, expected, 'RelTol', 1e-6);
    end

    function testPdfMultiVariate(testCase)
      dist = ExponentialDistribution(2, 3);
      x = [0.1 0.2 0.3; 1.35 2.35 3.35; 4 4 4];
      actual = dist.Pdf(x);
      expected = prod(exppdf(x, 2), 2);
      testCase.verifyEqual(actual, expected, 'RelTol', 1e-6);
    end

    function testLogPdf(testCase)
      dist = ExponentialDistribution(2);
      actual = dist.LogPdf(1.35);
      expected = log(exppdf(1.35, 2));
      testCase.verifyEqual(actual, expected, 'RelTol', 1e-6);
    end

    function testLogPdfMultiVariate(testCase)
      dist = ExponentialDistribution(2, 3);
      x = [0.1 0.2 0.3; 1.35 2.35 3.35; 4 4 4];
      actual = dist.LogPdf(x);
      expected = log(prod(exppdf(x, 2), 2));
      testCase.verifyEqual(actual, expected, 'RelTol', 1e-6);
    end

  end

end
