%%% Monte Carlo Dynamic Classifier tool - MCDC tool

% Copyright (C) 2014 Gareth W. Peters, Tomoko Matsui, Francois Septier, and Akira Tamamori

% This program is free software; you can redistribute it and/or modify it under the terms
% of the GNU General Public License as published by the Free Software Foundation; either
% version 2 of the License, or (at your option) any later version.

% This program is distributed in the hope that it will be useful, but WITHOUT ANY
% WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
% PARTICULAR PURPOSE.  See the GNU General Public License for more details.

% You should have received a copy of the GNU General Public License along with this
% program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
% Floor, Boston, MA 02110-1301 USA.

%
% LorenzModeltest.m
%

T      = 1000;
Sigma  = 10;
Rho    = 28;
Beta   = 8/3;
Alpha0 = 1;
q      = 1e-1;
r      = 1e-1;
dt     = 1e-2;

[x, y] = LorenzModel(T, Sigma, Rho, Beta, Alpha0, q, r, dt);

% Image settings

width  = 1024;
height = 768;
set(gcf, 'PaperPositionMode', 'auto')
pos = get(gcf, 'Position');
pos(3) = width - 1;
pos(4) = height;
set(gcf, 'Position', pos);

%
% Plotting
%

plot(1:T, 1:T);
print('-r0', '-dpng', 't_t.png');

% T-X
plot(1:T, x(:, 1));
print('-r0', '-dpng', 't_x1.png');
plot(1:T, x(:, 2));
print('-r0', '-dpng', 't_x2.png');

% T-Y
plot(1:T, y(:, 1));
print('-r0', '-dpng', 't_y1.png');
plot(1:T, y(:, 2));
print('-r0', '-dpng', 't_y2.png');
plot(1:T, y(:, 3));
print('-r0', '-dpng', 't_y3.png');

% X transition
plot(x(:, 1), x(:, 2));
print('-r0', '-dpng', 'x1_x2.png');

% Y transition
plot(y(:, 1), y(:, 2));
print('-r0', '-dpng', 'y1_y2.png');
plot(y(:, 1), y(:, 3));
print('-r0', '-dpng', 'y1_y3.png');

quit;
