%%% Monte Carlo Dynamic Classifier tool - MCDC tool

% Copyright (C) 2014 Gareth W. Peters, Tomoko Matsui, Francois Septier, and Akira Tamamori

% This program is free software; you can redistribute it and/or modify it under the terms
% of the GNU General Public License as published by the Free Software Foundation; either
% version 2 of the License, or (at your option) any later version.

% This program is distributed in the hope that it will be useful, but WITHOUT ANY
% WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
% PARTICULAR PURPOSE.  See the GNU General Public License for more details.

% You should have received a copy of the GNU General Public License along with this
% program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
% Floor, Boston, MA 02110-1301 USA.

%
% NormalDistributionTest.m
%
classdef NormalDistributionTest < matlab.unittest.TestCase

  methods (Test)

    function testNormalDistribution(testCase)
      dist = NormalDistribution(2, 3);
      testCase.verifyEqual(dist.Mean, 2);
      testCase.verifyEqual(dist.Variance, 3);
    end

    function testPdf(testCase)
      dist = NormalDistribution(2, 3);
      actual = dist.Pdf(0);
      expected = normpdf(0, 2, sqrt(3));
      testCase.verifyEqual(actual, expected, 'RelTol', 1e-6);
    end

    function testPdfMultiVariate(testCase)
      dist = NormalDistribution(2, 3, 3);
      x = [0 0 0; 1 2 3; 5 4 3];
      actual = dist.Pdf(x);
      expected = prod(normpdf(x, 2, sqrt(3)), 2);
      testCase.verifyEqual(actual, expected, 'RelTol', 1e-6);
    end

    function testLogPdf(testCase)
      dist = NormalDistribution(2, 3);
      actual = dist.LogPdf(0);
      expected = log(normpdf(0, 2, sqrt(3)));
      testCase.verifyEqual(actual, expected, 'RelTol', 1e-6);
    end

    function testLogPdfMultiVariate(testCase)
      dist = NormalDistribution(2, 3, 3);
      x = [0 0 0; 1 2 3; 5 4 3];
      actual = dist.LogPdf(x);
      expected = log(prod(normpdf(x, 2, sqrt(3)), 2));
      testCase.verifyEqual(actual, expected, 'RelTol', 1e-6);
    end

  end

end
