%%% Monte Carlo Dynamic Classifier tool - MCDC tool

% Copyright (C) 2014 Gareth W. Peters, Tomoko Matsui, Francois Septier, and Akira Tamamori

% This program is free software; you can redistribute it and/or modify it under the terms
% of the GNU General Public License as published by the Free Software Foundation; either
% version 2 of the License, or (at your option) any later version.

% This program is distributed in the hope that it will be useful, but WITHOUT ANY
% WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
% PARTICULAR PURPOSE.  See the GNU General Public License for more details.

% You should have received a copy of the GNU General Public License along with this
% program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
% Floor, Boston, MA 02110-1301 USA.

%
% TDistributionTest.m
%
classdef TDistributionTest < matlab.unittest.TestCase

  methods (Test)

    function testTDistribution(testCase)
      dist = TDistribution(1.5);
      testCase.verifyEqual(dist.Nu, 1.5);
    end

    function testPdf(testCase)
      dist = TDistribution(1.5);
      actual = dist.Pdf(0);
      expected = tpdf(0, 1.5);
      testCase.verifyEqual(actual, expected, 'RelTol', 1e-6);
    end

    function testPdfMultiVariate(testCase)
      dist = TDistribution(1.5, 3);
      x = [0 0 0; 1 2 3; 5 4 3];
      actual = dist.Pdf(x);
      expected = prod(tpdf(x, 1.5), 2);
      testCase.verifyEqual(actual, expected, 'RelTol', 1e-6);
    end

    function testLogPdf(testCase)
      dist = TDistribution(1.5);
      actual = dist.LogPdf(0);
      expected = log(tpdf(0, 1.5));
      testCase.verifyEqual(actual, expected, 'RelTol', 1e-6);
    end

    function testLogPdfMultiVariate(testCase)
      dist = TDistribution(1.5, 3);
      x = [0 0 0; 1 2 3; 5 4 3];
      actual = dist.LogPdf(x);
      expected = log(prod(tpdf(x, 1.5), 2));
      testCase.verifyEqual(actual, expected, 'RelTol', 1e-6);
    end

  end

end
