%%% Monte Carlo Dynamic Classifier tool - MCDC tool

% Copyright (C) 2014 Gareth W. Peters, Tomoko Matsui, Francois Septier, and Akira Tamamori

% This program is free software; you can redistribute it and/or modify it under the terms
% of the GNU General Public License as published by the Free Software Foundation; either
% version 2 of the License, or (at your option) any later version.

% This program is distributed in the hope that it will be useful, but WITHOUT ANY
% WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
% PARTICULAR PURPOSE.  See the GNU General Public License for more details.

% You should have received a copy of the GNU General Public License along with this
% program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
% Floor, Boston, MA 02110-1301 USA.

%
% UniformDistributionTest.m
%
classdef UniformDistributionTest < matlab.unittest.TestCase

  methods (Test)

    function testUniformDistribution(testCase)
      dist = UniformDistribution(-0.5, 3.5);
      testCase.verifyEqual(dist.LowerBound, -0.5);
      testCase.verifyEqual(dist.UpperBound, 3.5);
    end

    function testPdf(testCase)
      dist = UniformDistribution(-0.5, 3.5);
      actual = dist.Pdf(0);
      expected = 1/(3.5+0.5);
      testCase.verifyEqual(actual, expected, 'RelTol', 1e-6);
    end

    function testPdfLowerBound(testCase)
      dist = UniformDistribution(-0.5, 3.5);
      actual = dist.Pdf(-1);
      expected = 0;
      testCase.verifyEqual(actual, expected, 'RelTol', 1e-6);
    end

    function testPdfUpperBound(testCase)
      dist = UniformDistribution(-0.5, 3.5);
      actual = dist.Pdf(5);
      expected = 0;
      testCase.verifyEqual(actual, expected, 'RelTol', 1e-6);
    end

    function testPdfMultiVariate(testCase)
      dist = UniformDistribution(-0.5, 3.5, 3);
      x = [0 0 0; 1 2 3; 5 4 3];
      actual = dist.Pdf(x);
      expected = [1 1 0]' * (1/(3.5+0.5))^3;
      testCase.verifyEqual(actual, expected, 'RelTol', 1e-6);
    end

    function testLogPdf(testCase)
      dist = UniformDistribution(-0.5, 3.5);
      actual = dist.LogPdf(0);
      expected = log(1/(3.5+0.5));
      testCase.verifyEqual(actual, expected, 'RelTol', 1e-6);
    end

    function testLogPdfLowerBound(testCase)
      dist = UniformDistribution(-0.5, 3.5);
      actual = dist.LogPdf(-1);
      expected = -Inf;
      testCase.verifyEqual(actual, expected, 'RelTol', 1e-6);
    end

    function testLogPdfUpperBound(testCase)
      dist = UniformDistribution(-0.5, 3.5);
      actual = dist.LogPdf(5);
      expected = -Inf;
      testCase.verifyEqual(actual, expected, 'RelTol', 1e-6);
    end

    function testLogPdfMultiVariate(testCase)
      dist = UniformDistribution(-0.5, 3.5, 3);
      x = [0 0 0; 1 2 3; 5 4 3];
      actual = dist.LogPdf(x);
      expected = log([1 1 0]' * (1/(3.5+0.5))^3);
      testCase.verifyEqual(actual, expected, 'RelTol', 1e-6);
    end

  end

end
