%%% Monte Carlo Dynamic Classifier tool - MCDC tool
% http://www.ismvideo.org/NDE0UP/MCDCtool/en/index.html

% Copyright (C) 2014 Gareth W. Peters, Tomoko Matsui, Francois Septier, and Akira Tamamori

% This program is free software; you can redistribute it and/or modify it under the terms
% of the GNU General Public License as published by the Free Software Foundation; either
% version 2 of the License, or (at your option) any later version.

% This program is distributed in the hope that it will be useful, but WITHOUT ANY
% WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
% PARTICULAR PURPOSE.  See the GNU General Public License for more details.

% You should have received a copy of the GNU General Public License along with this
% program; if not, see <http://www.gnu.org/licenses/>.

%
% LinearStateSpaceModelPSMCEstimation.m
%

rng('default');

warning off

profile on
tic

%
% Generate state and observation time series from Linear state space model
%

T    = 5;
k1   = 0.1;
k2   = 0.1;
mass = 1.0;
q    = 0.1;
r    = 0.1;
dt   = 1.0;

[x, y, F, H] = LinearStateSpaceModel(T, k1, k2, mass, q, r, dt);

save('logs/LinearStasteSpaceModel_Data.mat');

grids = { ...
  [-2:0.1:2], ...
  [-2:0.1:2]  ...
};

stateKernelGens = { ...
  RBFKernelGenerator(UniformDistribution(0.01, 10), UniformDistribution(0.01, 10)), ...
  RBFKernelGenerator(UniformDistribution(0.01, 10), UniformDistribution(0.01, 10))  ...
};

obsKernelGens = {
  RBFKernelGenerator(UniformDistribution(0.01, 10), UniformDistribution(0.01, 10))  ...
};

stateMeanFuncs = { ...
  @(x) x(1,:), ...
  @(x) x(2,:)  ...
};

obsMeanFuncs = { ...
  @(x) x(1,:)  ...
};

x0 = zeros(1, 2);
xaux = []; % No auxiliary states
u = [];    % No controls

N  = 10;
J  = 10;

aspect = Aspect();
aspect.LogFileName = 'logs/LinearStateSpaceModelPSMC.log';
aspect.MatFileNamePrefix = 'logs/LinearStateSpaceModelPSMC';
aspect.SavesIntermediateMat = true;

disp('PSMC Estimation using SPAPI');

stateSplineHandle = @GenericSpline;
obsSplineHandle = @GenericSpline;
algorithm = PSMC();
algorithm.SetPMCMCProbability(1.0); % Don't use MH-Gibbs

stateModel = GaussianProcessModel(ModelKind.State, algorithm);
algorithm.SetStateModel(stateModel);

obsModel = GaussianProcessModel(ModelKind.Observation, algorithm);
algorithm.SetObsModel(obsModel);

likelihoodCalculator = ParticleFilter(x0, xaux, u, y, q, r, N, 0);
algorithm.SetLikelihoodCalculator(likelihoodCalculator);

[ IDX, SKP, OKP, FV, GV, XE, YE, loglik ] = MCDCTrain( ...
  algorithm, ...
  grids, ...
  stateKernelGens, ...
  obsKernelGens, ...
  stateMeanFuncs, ...
  obsMeanFuncs, ...
  0, ... % Not used in PSMC estimation
  stateSplineHandle, ...
  obsSplineHandle, ...
  x0, ...
  [], ... % No auxiliary states
  [], ... % No controls
  y', ...
  N, ...
  J, ...
  0, ...
  aspect ...
);

time = toc;

save('logs/LinearStateSpaceModelPSMC.mat');

disp('Done');

profile viewer
p = profile('info');
%profsave(p,'logs/profile_results_Linear_Unit_Test')
%profsave(p, fullfile(mfilename, 'logs', 'profile_results_Linear_Unit_Test'))

disp('Total Compute Time (sec)')
time
